package net.wanji.opt.servicev2.judgeanalysis.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.dao.entity.GreenwaveCrossPO;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.GreenwaveCrossMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemCrossDayMapper;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemCrossDay;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemGreenDay;
import net.wanji.opt.entity.judgeanalysis.CrossPoint;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemCrossDayService;
import net.wanji.opt.synthesis.pojo.CrossOptAnalysisEntity;
import net.wanji.opt.vo2.CrossOptAnalysisVO;
import net.wanji.opt.vo2.GreenwaveCrossResult;
import net.wanji.opt.vo2.JudTrunkLineCrossProblemEntityVO;
import net.wanji.opt.vo2.LaneSturationInfo;
import net.wanji.opt.vo2.report.JudTrunkLineCrossProblemEntityDTO;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * <p>
 * 路口问题分析 服务实现类
 * </p>
 *
 * @author fengyi
 * @since 2025-03-13
 */
@Service
public class AnalysisProblemCrossDayServiceImpl extends ServiceImpl<AnalysisProblemCrossDayMapper, AnalysisProblemCrossDay> implements AnalysisProblemCrossDayService {

    @Resource
    private AnalysisProblemCrossDayMapper analysisProblemCrossDayMapper;
    @Autowired
    private GreenwaveCrossMapper greenwaveCrossMapper;
    @Autowired
    private BaseCrossInfoMapper baseCrossInfoMapper;


    /**
     * 路口问题分析详情
     *
     * @param id
     * @return
     */
    @Override
    public AnalysisProblemCrossDay info(Long id) {

        return getById(id);
    }

    /**
     * 路口问题分析新增
     *
     * @param param 根据需要进行传值
     * @return
     */
    @Override
    public void add(AnalysisProblemCrossDay param) {

        save(param);
    }

    /**
     * 路口问题分析修改
     *
     * @param param 根据需要进行传值
     * @return
     */
    @Override
    public void modify(AnalysisProblemCrossDay param) {

        updateById(param);
    }

    /**
     * 路口问题分析删除(单个条目)
     *
     * @param id
     * @return
     */
    @Override
    public void remove(Long id) {
        removeById(id);
    }

    /**
     * 路口问题分析删除(多个条目)
     *
     * @param ids
     * @return
     */
    @Override
    public void removes(List<Long> ids) {

        removeByIds(ids);
    }

    /**
     * 路口区域id数据添加
     *
     * @return
     */
    @Override
    public void updateCrossAreaIdByCondition() {
        //判断路口的areaId是否存在
        List<CrossPoint> crossPoints = analysisProblemCrossDayMapper.findCrossPoint();
        if (crossPoints.size() > 0) {
            //获取区域id的坐标范围
            List<CrossPoint> areaPont = analysisProblemCrossDayMapper.findAreaPoint();
            //循环判断是否存在区域内
            for (CrossPoint crossPoint : crossPoints) {
                int i = 0;
                for (CrossPoint areaPoint : areaPont) {
                    String a = crossPoint.getPoint().replaceAll(" ", ",");
                    Geometry geo = net.wanji.common.gts.GeomsConvertUtil.createPoint(a);
                    Geometry geo1 = net.wanji.common.gts.GeomsConvertUtil.genGeometry(areaPoint.getPoint());
                    if (geo.within(geo1)) {
                        //如果匹配则更新id
                        analysisProblemCrossDayMapper.updateCrossAreaIdByid(areaPoint.getId(), crossPoint.getId());
                        i = 1;
                    }
                }
                if (i == 0) {
                    //未匹配则把区域id设置为0
                    analysisProblemCrossDayMapper.updateCrossAreaIdByid("0", crossPoint.getId());
                }
            }
        }

    }

    @Override
    public List<AnalysisProblemCrossDay> getListSingleByStartAndEnd(String crossId, Integer startTime, Integer endTime) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        map.put("crossId", crossId);
        return analysisProblemCrossDayMapper.getListSingleByStartAndEnd(map);
    }

    public List<AnalysisProblemCrossDay> selectCountByType(Map<String, Object> map) {
        return analysisProblemCrossDayMapper.selectCountByType(map);
    }

    Map<String, Object> getParamMap(String crossId, Integer startTime, Integer endTime) {
        Map<String, Object> map = new HashMap();
        map.put("crossId", crossId);
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        return map;
    }

    @Override
    public Map<String, Object> selectSingerByProblemTrend(String crossId, Integer startTime, Integer endTime) throws ParseException {
        List<AnalysisProblemCrossDay> list = analysisProblemCrossDayMapper.selectSingerByProblemTrend(getParamMap(crossId, startTime, endTime));
        Date sd = DateUtil.parse(startTime.toString(), EsDateIndexUtil.YMD_FORMATTER);
        Date ed = DateUtil.parse(endTime.toString(), EsDateIndexUtil.YMD_FORMATTER);
        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis("4", DateUtil.formatDate(sd, EsDateIndexUtil.YMD_HM_FORMATTER), DateUtil.formatDate(ed, EsDateIndexUtil.YMD_HM_FORMATTER));
        List<Map<String, Object>> allList = new ArrayList<>();

        Map<String, List<AnalysisProblemCrossDay>> groupByType = list.stream().collect(Collectors.groupingBy(o -> o.getEventType(), TreeMap::new, Collectors.toList()));

        for (Map.Entry<String, List<AnalysisProblemCrossDay>> entry : groupByType.entrySet()) {
            String key = entry.getKey();
            //同一类事件每个日期问题数量
            List<AnalysisProblemCrossDay> value = entry.getValue();

            String typeName = value.get(0).getTypeName();

            // 构造一个Map用于快速查找已有数据点
            Map<String, Integer> existingDataPoints = value.stream()
                    .collect(Collectors.toMap(po -> po.getDt().toString(), AnalysisProblemCrossDay::getEventNumber));
            // 构造一个Map用于快速查找时长已有数据点
            Map<String, Integer> existingDate = value.stream()
                    .collect(Collectors.toMap(po -> po.getDt().toString(), AnalysisProblemCrossDay::getEventTotalTime));
            // 对于每个时间点，如果不存在数据则补0
            List<Integer> numberlist = sortedSet.stream()
                    .map(timePoint -> existingDataPoints.getOrDefault(timePoint, 0))
                    .collect(Collectors.toList());
            // 对于每个时间点，如果时长不存在数据则补0
            List<Integer> durationList = sortedSet.stream()
                    .map(timePoint -> existingDate.getOrDefault(timePoint, 0))
                    .collect(Collectors.toList());
            Map<String, Object> maplist = new HashMap<>();
            maplist.put("eventType", key);
            maplist.put("eventLabel", typeName);
            maplist.put("list", numberlist);
            maplist.put("durationList", durationList);
            allList.add(maplist);
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        return jsonObject;
    }

    @Override
    public CrossOptAnalysisVO getCrossOptAnalysis(String crossID, String startTime, String endTime) {
        List<CrossOptAnalysisEntity> entityList = analysisProblemCrossDayMapper.getCrossOptAnalysis(null, crossID, startTime, endTime);
        CrossOptAnalysisVO crossOptAnalysisVO = new CrossOptAnalysisVO();

        for (CrossOptAnalysisEntity c : entityList) {
            if (c.getStatus().equals("已优化")) {
                crossOptAnalysisVO.setOptimized(c.getNum());
            } else if (c.getStatus().equals("未优化")) {
                crossOptAnalysisVO.setUnoptimized(c.getNum());
            }
        }
        crossOptAnalysisVO.setTotal(crossOptAnalysisVO.getOptimized() + crossOptAnalysisVO.getUnoptimized());

        return crossOptAnalysisVO;
    }

    @Override
    public List<JudTrunkLineCrossProblemEntityVO> getTrunkLineCrossProblem(Integer greenId, Integer status, String startTime, String endTime) {
        List<GreenwaveCrossResult> greenwaveCrossResultList = analysisProblemCrossDayMapper.getTrunkLineCrossProblem(greenId, status, startTime, endTime);
        if (ObjectUtil.isEmpty(greenwaveCrossResultList)) {
            return null;
        }
        Map<Integer, List<GreenwaveCrossResult>> greenIdMap = greenwaveCrossResultList.stream().collect(Collectors.groupingBy(GreenwaveCrossResult::getGreenId));
        List<JudTrunkLineCrossProblemEntityDTO> crossProblemEntityDTOS = new ArrayList<>();
        for (List<GreenwaveCrossResult> greenwaveCrossResults : greenIdMap.values()) {
            Map<String, List<GreenwaveCrossResult>> crossIdMap = greenwaveCrossResults.stream().collect(Collectors.groupingBy(GreenwaveCrossResult::getCrossId));
            int slowCount = greenwaveCrossResults.stream().filter(x -> 2 == x.getStatus()).mapToInt(GreenwaveCrossResult::getDirCount).sum();
            int conCount = greenwaveCrossResults.stream().filter(x -> 3 == x.getStatus() || 4 == x.getStatus()).mapToInt(GreenwaveCrossResult::getDirCount).sum();
            List<GreenwaveCrossPO> greenwaveCrossPOS = greenwaveCrossMapper.selectByGreenwaveId(greenId);
            for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOS) {
                BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(greenwaveCrossPO.getCrossId());
                JudTrunkLineCrossProblemEntityDTO judTrunkLineCrossProblemEntityDTO = new JudTrunkLineCrossProblemEntityDTO();
                judTrunkLineCrossProblemEntityDTO.setGreenId(greenwaveCrossPO.getGreenId());
                judTrunkLineCrossProblemEntityDTO.setCrossId(greenwaveCrossPO.getCrossId());
                judTrunkLineCrossProblemEntityDTO.setCrossName(baseCrossInfoPO.getName());
                judTrunkLineCrossProblemEntityDTO.setSort(greenwaveCrossPO.getSort());
                judTrunkLineCrossProblemEntityDTO.setInDir(greenwaveCrossPO.getInDir());
                judTrunkLineCrossProblemEntityDTO.setOutDir(greenwaveCrossPO.getOutDir());
                judTrunkLineCrossProblemEntityDTO.setStatusR(1);
                judTrunkLineCrossProblemEntityDTO.setStatusL(1);
                List<GreenwaveCrossResult> crossResults = crossIdMap.get(greenwaveCrossPO.getCrossId());
                if (ObjectUtil.isNotEmpty(crossResults)) {
                    judTrunkLineCrossProblemEntityDTO.setCrossName(crossResults.get(0).getName());
                    if (ObjectUtil.isEmpty(status) || status == 2) {
                        int slowCountCross = crossResults.stream().filter(x -> 2 == x.getStatus()).mapToInt(GreenwaveCrossResult::getDirCount).sum();
                        if (slowCount > 0 && slowCountCross * 100 / slowCount > 10) {
                            Map<Integer, Integer> dirTypeCountMap = crossResults.stream().filter(x -> 2 == x.getStatus()).collect(Collectors.groupingBy(
                                    GreenwaveCrossResult::getDirType,
                                    Collectors.summingInt(GreenwaveCrossResult::getDirCount)
                            ));
                            for (Integer dirType : dirTypeCountMap.keySet()) {
                                if (dirType == crossResults.get(0).getInDir()) {
                                    judTrunkLineCrossProblemEntityDTO.setStatusL(2);
                                }
                                if (dirType == crossResults.get(0).getOutDir()) {
                                    judTrunkLineCrossProblemEntityDTO.setStatusR(2);
                                }
                            }
                        }
                    }
                }
                if (ObjectUtil.isEmpty(status) || status == 3 || status == 4) {
                    int conCountCross = crossResults.stream().filter(x -> 3 == x.getStatus() || 4 == x.getStatus()).mapToInt(GreenwaveCrossResult::getDirCount).sum();
                    if (conCount > 0 && conCountCross * 100 / conCount > 10) {
                        Map<Integer, Integer> dirTypeCountMap = crossResults.stream().collect(Collectors.groupingBy(
                                GreenwaveCrossResult::getDirType,
                                Collectors.summingInt(GreenwaveCrossResult::getDirCount)
                        ));
                        for (Integer dirType : dirTypeCountMap.keySet()) {
                            if (dirType == crossResults.get(0).getInDir()) {
                                judTrunkLineCrossProblemEntityDTO.setStatusL(3);
                            }
                            if (dirType == crossResults.get(0).getOutDir()) {
                                judTrunkLineCrossProblemEntityDTO.setStatusR(3);
                            }
                        }
                    }
                }
                crossProblemEntityDTOS.add(judTrunkLineCrossProblemEntityDTO);
            }
        }
        List<JudTrunkLineCrossProblemEntityVO> judTrunkLineCrossProblemEntityVOS = crossProblemEntityDTOS.stream().map(x -> {
            JudTrunkLineCrossProblemEntityVO judTrunkLineCrossProblemEntityVO = new JudTrunkLineCrossProblemEntityVO();
            BeanUtils.copyProperties(x, judTrunkLineCrossProblemEntityVO);
            return judTrunkLineCrossProblemEntityVO;
        }).collect(Collectors.toList());

        return judTrunkLineCrossProblemEntityVOS;
    }

    @Override
    public JSONObject getGreenCongestionAnalysis(Integer greenId, String startTime, String endTime) {
        Double stoptimes = analysisProblemCrossDayMapper.getGreenStopTimeAvg(greenId, startTime, endTime);
        Double stoptimesSum = analysisProblemCrossDayMapper.getGreenStopTimeSum(greenId, startTime, endTime);
        List<GreenwaveCrossPO> greenwaveCrossPOS = greenwaveCrossMapper.selectByGreenwaveId(greenId);
        JSONObject jsonObject = new JSONObject();
        if (stoptimesSum > greenwaveCrossPOS.size()) {
            jsonObject.put("slowAnalysis", "干线相位协调不佳，干线频繁停车，平均停车次数" + stoptimes + "次，易拥堵");
        }
        String crossName = "";
        for (GreenwaveCrossPO greenwaveCrossPO : greenwaveCrossPOS) {
            List<LaneSturationInfo> sturationInfoList = analysisProblemCrossDayMapper.getSturationInfoList(greenwaveCrossPO.getCrossId(), startTime, endTime);
            List<LaneSturationInfo> infoList = sturationInfoList.stream().filter(x -> greenwaveCrossPO.getInDir() == x.getDir() || greenwaveCrossPO.getOutDir() == x.getDir()).collect(Collectors.toList());
            if (ObjectUtil.isEmpty(infoList) || infoList.size() == 0) {
                continue;
            }
            List<LaneSturationInfo> sturationInfos = infoList.stream().filter(x -> ObjectUtil.isNotEmpty(x.getSturation()) && x.getSturation() > 0.8).collect(Collectors.toList());
            if (sturationInfos.size() * 1.0 / infoList.size() > 0.5) {
                BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(greenwaveCrossPO.getCrossId());
                if (ObjectUtil.isEmpty(crossName)) {
                    crossName = baseCrossInfoPO.getName();
                } else {
                    crossName = crossName + "、" + baseCrossInfoPO.getName();
                }
            }
            if (ObjectUtil.isNotEmpty(crossName)) {
                jsonObject.put("crossAnalysis", crossName + "协调方向流量饱和，易引发拥堵");
            }
        }
        return jsonObject;
    }

    @Override
    public JSONObject getGreenSlowAnalysis(Integer greenId, String startTime, String endTime) {
        Double stoptimes = analysisProblemCrossDayMapper.getGreenSlowStopTimeSum(greenId, startTime, endTime);
        List<GreenwaveCrossPO> greenwaveCrossPOS = greenwaveCrossMapper.selectByGreenwaveId(greenId);
        JSONObject jsonObject = new JSONObject();
        if (stoptimes > greenwaveCrossPOS.size() / 2) {
            jsonObject.put("slowAnalysis", "干线绿波协调不连续，易引发缓行");
        }
        return jsonObject;
    }

}
